=head1 NAME

Tails::MirrorPool - class that represents the Tails HTTP download mirror pool

=cut

package Tails::MirrorPool;

use Moose;
use Moose::Util::TypeConstraints qw{class_type};
use MooseX::Method::Signatures;
use MooseX::Types::Moose qw( :all );
use MooseX::Has::Sugar::Saccharin;

use 5.10.1;
use namespace::autoclean;
use warnings FATAL => 'all';
use autodie qw(:all);

use Carp;
use Carp::Assert;
use Carp::Assert::More;

use IPC::System::Simple qw{capturex};
use Tails::Download::HTTPS;

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
textdomain("tails-perl5lib");



=head1 TYPES

=cut

class_type 'Tails::Download::HTTPS';


=head1 ATTRIBUTES

=cut

has 'baseurl' => required ro Str;

has 'fallback_prefix' => lazy_build ro Str;

has 'downloader' =>
    lazy_build ro 'Tails::Download::HTTPS';

has 'filename' => lazy_build ro Str;


=head1 CONSTRUCTORS AND BUILDERS

=cut

method _build_fallback_prefix {
    q{http://dl.amnesia.boum.org/tails'}
}

method _build_downloader {
    Tails::Download::HTTPS->new();
}

method _build_filename {
    q{mirrors.json}
}


=head1 METHODS

=cut

method transformURL (Str $url) {
    my $orig_url = $url;
    my $mirrors_json = $self->downloader->get_url(
        $self->baseurl . '/' . $self->filename
    );
    $ENV{NODE_PATH} //= '/usr/local/lib/nodejs';
    $url = capturex(
        'tails-transform-mirror-url', $url, $self->fallback_prefix,
        $mirrors_json
    );
    say STDERR "Transformed '$orig_url' into '$url'" if $ENV{DEBUG};
    return $url;
}

no Moose;
1;
